local addonname, didi = ...

-- Get quest data from namespace map
local questData = didi._questdb
local reverseQuestData = didi._reverse

-- Store quests we already printed to avoid double-printing
local alreadyPrinted = {}

function didi:getPrerequisites(id)
    local last = 0
    for _, k in pairs(questData[tonumber(id)][5]) do
        last = k
    end
    if last ~= 0 then
        self:getPrerequisites(last)
    end

    for _, k in pairs(questData[tonumber(id)][5]) do
        if (not didi.tableContains(alreadyPrinted, k)) then
            self:printCompletionStatus(k)
            table.insert(alreadyPrinted, k)
        end
    end
end

function didi:getFollowUps(id)
    for _, k in pairs(questData[tonumber(id)][6]) do
        self:printCompletionStatus(k)
    end
end

function didi:printCompletionStatus(id)
    local questName
    if (didi.tableHasKey(questData, tonumber(id))) then
        questName = questData[tonumber(id)][1]
    else
        questName = "|cffff0000MISSING_QUEST_NAME|r"
    end

    local completed = C_QuestLog.IsQuestFlaggedCompleted(tonumber(id))
    if (completed) then
        print(tonumber(id) .. ": " .. questName .. ":|cff10ff00 You did it!! :D")
    else
        print(tonumber(id) .. ": " .. questName .. ": You didn't do it! :(")
    end
end

function didi:fuzzyMatch(term)
    if type(term) == "string" and term:len() >= 3 then
        term = string.lower(term)
        print('|cff10ff00Did I Do It? :: Searching for|r "' .. term .. '"')

        for name, data in pairs(reverseQuestData) do
            if string.find(name, term, 1, true) then
                print("|cff8ded74" .. questData[data[1]][1] .. ":|r ", table.concat(data, ", "))
            end
        end
    end
end

SLASH_DIDI1 = "/didi"
SLASH_DIDI2 = "/dididoit"
SlashCmdList["DIDI"] = function(msg)
    if (tonumber(msg) ~= nil and string.len(msg) > 0) then
        print("|cff10ff00 Did I Do It? :: Completion status for " .. msg)

        if didi.tableHasKey(questData, tonumber(msg)) then
            didi:getPrerequisites(msg)
            alreadyPrinted = {}

            didi:printCompletionStatus(msg)

            didi:getFollowUps(msg)
            alreadyPrinted = {}
        else
            didi:printCompletionStatus(msg)
            print("        |cffff0000Missing data... please report this quest ID on the addon page at")
            print("        |rhttps://www.curseforge.com/wow/addons/did-i-do-it")
        end
    elseif type(msg) == "string" and msg:len() >= 3 then
        didi:fuzzyMatch(msg)
    else
        print("|cffbaff00 Did I Do It? :: |cffd91d14Bad request: " .. msg)
        print("Check Completion Syntax: /didi {Quest ID}")
        print("Search Quest Name: /didi {search} (minimum 3 letters)")
    end
end
